package rpc

import (
	"context"
	"encoding/json"

	"git.loafle.net/commons_go/rpc"
	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol"
	crpj "git.loafle.net/commons_go/rpc/protocol/json"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
	"google.golang.org/grpc/metadata"
)

func newGatewayRPCServletHandler() GatewayRPCServletHandler {
	sh := &GatewayRPCServletHandlers{}
	sh.RegisterCodec(crpj.Name, crpj.NewServerCodec())
	return sh
}

type GatewayRPCServletHandlers struct {
	gateway.ServletHandlers
}

func (sh *GatewayRPCServletHandlers) Invoke(servletCTX rpc.ServletContext, requestCodec protocol.RegistryCodec) (result interface{}, err error) {
	md := servletCTX.GetAttribute(oogw.GRPCMetadataKey)
	if nil == md {
		md = metadata.Pairs(
			oogw.GRPCClientTypeKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string),
			oogw.GRPCSessionIDKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string),
			oogw.GRPCTargetIDKey, servletCTX.GetAttribute(oogw.SocketIDKey).(string))

		servletCTX.SetAttribute(oogw.GRPCMetadataKey, md)
	}

	grpcCTX := metadata.NewOutgoingContext(context.Background(), md.(metadata.MD))

	params, err := requestCodec.Params()
	if nil != err {
		return nil, err
	}

	r, err := grpc.Exec(grpcCTX, requestCodec.Method(), params)
	if err != nil {
		return nil, err
	}

	var m interface{}
	err = json.Unmarshal([]byte(r), &m)
	if err != nil {
		return nil, err
	}

	return m, nil
}

func (sh *GatewayRPCServletHandlers) Validate() {
	sh.ServletHandlers.Validate()

}
